<?php

/**
 * Dynamic Ads + Pixel
 *
 * @author    businesstech.fr <modules@businesstech.fr> - https://www.businesstech.fr/
 * @copyright Business Tech - https://www.businesstech.fr/
 * @license   see file: LICENSE.txt
 *
 *           ____    _______
 *          |  _ \  |__   __|
 *          | |_) |    | |
 *          |  _ <     | |
 *          | |_) |    | |
 *          |____/     |_|
 */
if (!defined('_PS_VERSION_')) {
    exit;
}

use FacebookProductAd\Dao\moduleDao;
use FacebookProductAd\Models\googleTaxonomy;
use FacebookProductAd\ModuleLib\moduleTools;
use FacebookProductAd\Models\categoryTaxonomy;
use FacebookProductAd\Configuration\moduleConfiguration;

class AdminTaxonomyController extends \ModuleAdminController
{
    /**
     * init content
     *
     * @since 1.5.0
     *
     * @return html
     */
    public function initContent()
    {
        parent::initContent();

        $isoLang = \Tools::getValue('sLangIso');

        if (!empty($isoLang)) {
            $isoExplode = explode('-', \Tools::getValue('sLangIso'));
            $id_lang = \Language::getIdByIso($isoExplode[0]);

            $jsDefs = [];
            $jsDefs['taxonomyController'] = $this->context->link->getAdminLink('AdminTaxonomy') . '&iLangId=' . $id_lang . '&sLangIso=' . $isoLang;
            \Media::addJsDef(['btFbda' => $jsDefs]);

            $shopCategories = moduleDao::getShopCategories(\FacebookProductAd::$iShopId, (int)$id_lang, \FacebookProductAd::$conf['FPA_HOME_CAT_ID'], \FacebookProductAd::$conf['FPA_HOME_CAT']);

            foreach ($shopCategories as &$category) {
                // get google taxonomy
                $aGoogleCat = categoryTaxonomy::getFacebookCategories(\FacebookProductAd::$iShopId, $category['id_category'], $isoLang, moduleConfiguration::FPA_TABLE_PREFIX);
                // assign the current taxonomy
                $category['google_category_name'] = is_array($aGoogleCat) && isset($aGoogleCat['txt_taxonomy']) ? $aGoogleCat['txt_taxonomy'] : '';
            }

            $this->context->smarty->assign([
                'moduleUrl' => \Context::getContext()->link->getAdminLink('AdminModules') . '&configure=facebookproductad&tab=taxonomies',
                'idLang' => $id_lang,
                'isoLang' => $isoLang,
                'currencyIso' => \Language::getIsoById(\FacebookProductAd::$iCurrentLang),
                'maxPostVar' => ini_get('max_input_vars'),
                'shopCategories' => $shopCategories,
                'shopCategoriesCount' => count($shopCategories),
                'faqLink' => 'http://faq.businesstech.fr',
                'taxonomiesToImport' => moduleTools::getTaxonomiesToImport($isoLang),
                'sModuleName' => moduleConfiguration::FPA_MODULE_SET_NAME,
            ]);

            // execute the ajax call
            if (\Tools::getValue('action') == 'autocomplete') {
                $this->processAutocomplete();
            }

            $this->context->smarty->assign([
                'content' => $this->content . $this->module->fetch('module:facebookproductad/views/templates/admin/tab/taxonomies.tpl'),
            ]);
        } else {
            \Tools::redirect(\Context::getContext()->link->getAdminLink('AdminModules') . '&configure=facebookproductad&tab=taxonomies');
        }
    }

    /**
     * manages to initialize controller's media
     *
     * @param bool $isNewTheme
     *
     * @return string
     */
    public function setMedia($isNewTheme = false)
    {
        parent::setMedia();

        $this->addCss(_MODULE_DIR_ . $this->module->name . '/views/css/admin.css');
        $this->addCss(_MODULE_DIR_ . $this->module->name . '/views/css/taxonomie.css');
        $this->addCss(_MODULE_DIR_ . $this->module->name . '/views/css/bootstrap4.css');
        $this->addJS(_MODULE_DIR_ . $this->module->name . '/views/js/module.js');
        $this->addJS(_MODULE_DIR_ . $this->module->name . '/views/js/taxonomies.js');
    }

    /**
     * Post process
     *
     * @since 1.5.0
     *
     * @return void
     */
    public function postProcess()
    {
        $isoLang = \Tools::getValue('sLangIso');

        if (Tools::isSubmit('save_btn')) {
            try {
                $iso = explode('-', \Tools::getValue('sLangIso'));
                $facebookCategories = \Tools::getValue('bt_facebook-cat');

                // delete previous facebook matching categories
                if (categoryTaxonomy::deleteFacebookCategory(\FacebookProductAd::$iShopId, $isoLang, moduleConfiguration::FPA_TABLE_PREFIX)) {
                    foreach ($facebookCategories as $idShopCategorie => $facebookCategoryValue) {
                        if (!empty($facebookCategoryValue)) {
                            // insert each category
                            categoryTaxonomy::insertFacebookCategory(\FacebookProductAd::$iShopId, $idShopCategorie, $facebookCategoryValue, $isoLang, moduleConfiguration::FPA_CAT_TAXO);
                        }
                    }
                }

                $this->confirmations[] = $this->module->l('Your matching Facebook categories have been updated');
            } catch (\Exception $e) {
                $this->errors[] = $e->getMessage();
            }
        }

        if (Tools::isSubmit('gmcTaxonomies') || Tools::isSubmit('gmcpTaxonomies') || Tools::isSubmit('tkpTaxonomies')) {
            try {
                $moduleSource = '';

                if (!empty(Tools::isSubmit('gmcTaxonomies'))) {
                    $moduleSource = 'gmcTaxonomies';
                } elseif (!empty(Tools::isSubmit('gmcpTaxonomies'))) {
                    $moduleSource = 'gmcpTaxonomies';
                } elseif (!empty(Tools::isSubmit('tkpTaxonomies'))) {
                    $moduleSource = 'tkpTaxonomies';
                }

                if (!empty($moduleSource)) {
                    $dataToImport = moduleTools::getTaxonomiesToImport($isoLang);

                    if (categoryTaxonomy::deleteFacebookCategory(\FacebookProductAd::$iShopId, $isoLang, moduleConfiguration::FPA_TABLE_PREFIX)) {
                        if (!empty($dataToImport)) {
                            foreach ($dataToImport[$moduleSource] as $data) {
                                categoryTaxonomy::insertFacebookCategory(\FacebookProductAd::$iShopId, $data['id_category'], $data['txt_taxonomy'], $data['lang'], moduleConfiguration::FPA_CAT_TAXO);
                            }
                        }
                    }
                    $this->confirmations[] = $this->module->l('Your matching Facebook categories have been imported');
                }
            } catch (\Exception $e) {
                $this->errors[] = $e->getMessage();
            }
        }
    }

    /**
     * manages the search customer autcomplete
     *
     * @return string
     */
    public function processAutocomplete()
    {
        $items = [];
        $isoLang = \Tools::getValue('sLangIso');
        $query = \Tools::getValue('query');
        $words = explode(' ', $query);
        $taxonomyFound = [];

        if (strlen($query) >= 4) {
            $items = googleTaxonomy::autocompleteSearch($isoLang, $words, moduleConfiguration::FPA_TABLE_PREFIX);
            if (!empty($items) && is_array($items)) {
                foreach ($items as $data) {
                    $taxonomyFound[] = $data['value'];
                }
            }
        }

        die(json_encode($taxonomyFound));
    }
}
