<?php

/**
 * Dynamic Ads + Pixel
 *
 * @author    BusinessTech.fr
 * @copyright Business Tech
 * @license   Commercial
 *
 *           ____    _______
 *          |  _ \  |__   __|
 *          | |_) |    | |
 *          |  _ <     | |
 *          | |_) |    | |
 *          |____/     |_|
 */

use FacebookProductAd\ModuleLib\moduleTools;
use FacebookProductAd\FacebookApi\FacebookClient;

class facebookproductadAjaxModuleFrontController extends ModuleFrontController
{
    public $ssl = true;
    protected $jsonOutput = [];
    public $ajax = true;

    /**
     * init module front controller
     */
    public function init()
    {
        // exec parent
        parent::init();
        $this->ajax = true;
    }

    /**
     * set the ajax
     */
    public function displayAjax()
    {
        $sAction = Tools::getValue('action', 'undefined');

        if (!empty($sAction) && method_exists($this, 'ajaxProcess' . Tools::toCamelCase($sAction))) {
            $this->{'ajaxProcess' . Tools::toCamelCase($sAction)}();
        } else {
            $this->errors[] = $this->module->l('Undefined action', 'ajax');
        }
    }

    /**
     * handle ajax for handle the select promotion clicked
     */
    protected function ajaxProcessUpdateConsent()
    {
        $sToken = Tools::getValue('token');
        $sModuleToken = Tools::getToken(false);
        // Do not execute if token is missing or false
        if (!empty($sToken) && $sModuleToken == $sToken) {
            // Set the user cookie
            Context::getContext()->cookie->bt_fbda_consent_lvl = 3;

            die(\json_encode(true));
        }
    }

    /**
     * handle ajax to manage the conversion API call
     */
    protected function ajaxProcessSendApiData()
    {
        $sToken = Tools::getValue('token');
        $sModuleToken = Tools::getToken(false);
        $tagContent = Tools::getValue('tagContent');
        $sendPageViewViaApi = Tools::getValue('useApiForPageView');
        // Do not execute if token is missing or false
        if (!empty($sToken) && $sModuleToken == $sToken) {
            if (!empty($tagContent)) {
                // Use case for product type that needs handle some data
                if (isset($tagContent['aDynTags']['content_type'])) {
                    if ($tagContent['aDynTags']['content_type']['value'] == 'product') {
                        $priceData = [
                            'currency' => isset($tagContent['aDynTags']['currency']['value']) ? $tagContent['aDynTags']['currency']['value'] : '',
                            'value' => isset($tagContent['aDynTags']['value']['value']) ? $tagContent['aDynTags']['value']['value'] : '',
                        ];
                        $content_category = isset($tagContent['aDynTags']['content_category']) ? $tagContent['aDynTags']['content_category']['value'] : '';
                        $content_name = isset($tagContent['aDynTags']['content_name']) ? $tagContent['aDynTags']['content_name']['value'] : '';
                        $content_ids = isset($tagContent['aDynTags']['content_ids']) ? $tagContent['aDynTags']['content_ids']['value'] : '';
                        $tracking_type = isset($tagContent['aTrackingType']) ? $tagContent['aTrackingType']['value'] : '';
                        $user_data = moduleTools::getApiUserData(\Context::getContext(), $tracking_type, 'product', $content_category, $content_ids, $content_name, false, $priceData);
                        FacebookClient::send($user_data);

                        if (!empty($sendPageViewViaApi)) {
                            $user_data = moduleTools::getApiUserData(\Context::getContext(), 'PageView', '', '', '', '', false, '');
                            FacebookClient::send($user_data);
                        }
                    }
                }
            }
        }
    }

    /**
     * handle ajax for handle the select promotion clicked
     */
    protected function ajaxProcessAddToCart()
    {
        $token = Tools::getValue('token');
        $module_token = Tools::getToken(false);
        // Do not execute if token is missing or false
        if (!empty($token) && $module_token == $token) {
            $id_product = Tools::getValue('id_product');
            $product = new Product((int)$id_product, FacebookProductAd::$iCurrentLang);
            $price = Product::getPriceStatic((int)$product->id, true, false, 2, null, false, true);
            $lang_prefix = new \Language(\Context::getContext()->cookie->id_lang);

            if (!empty($id_product)) {
                $this->jsonOutput['content_type'] = 'product';
                $this->jsonOutput['content_ids'] = moduleTools::buildContentIds('product', $lang_prefix->iso_code, $id_product);
                $this->jsonOutput['value'] = number_format($price, 2, '.', ',');
                $this->jsonOutput['currency'] = Context::getContext()->currency->iso_code;
            }

            die(json_encode($this->jsonOutput));
        }
    }

    /**
     * handle the ajax on product combination update
     */
    protected function ajaxProcessUpdateCombination()
    {
        $sToken = \Tools::getValue('token');
        $sModuleToken = \Tools::getToken(false);

        // Do not execute if token is missing or false
        if (!empty($sToken) && $sModuleToken == $sToken) {
            $id_product_attribute = \Tools::getValue('id_product_attribute');
            $id_product = \Tools::getValue('id_product');
            $language = new \Language(\Context::getContext()->cookie->id_lang);
            $prefix_lang = !empty(\FacebookProductAd::$conf['FPA_ADD_LANG_ID']) ? \Tools::strtoupper($language->iso_code) : '';
            $prefix_id = \Tools::strtoupper(\FacebookProductAd::$conf['FPA_ID_PREFIX']);

            if (!empty($id_product)) {
                $oProduct = new \Product((int)$id_product, \FacebookProductAd::$iCurrentLang);
                $oCategory = new \Category((int)$oProduct->id_category_default, \FacebookProductAd::$iCurrentLang);
                $fPrice = \Product::getPriceStatic((int)$oProduct->id, true, false, 2, null, false, true);

                $this->jsonOutput['content_id'] = moduleTools::buildContentIds('product', 'FR', $id_product);
                $this->jsonOutput['content_name'] = str_replace('\'', '', $oProduct->name[\FacebookProductAd::$iCurrentLang]);
                $this->jsonOutput['currency'] = \Context::getContext()->currency->iso_code;
                $this->jsonOutput['content_category'] = $oCategory->name;
                $this->jsonOutput['value'] = number_format($fPrice, 2, '.', ',');
            }

            die(json_encode($this->jsonOutput));
        }
    }
}
